'use strict';

var _underscore = require('underscore.string');

var _underscore2 = _interopRequireDefault(_underscore);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Returns human readable form
 * @param {String} name - name to convert
 * @return {String} - converted name
 */
var humanName = function humanName(name) {
  return _underscore2.default.humanize(name);
};

/**
 * Returns hyphenated name
 * @param {String} name - name to convert
 * @return {String} - converted name
 */
var hyphenName = function hyphenName(name) {
  return _underscore2.default.slugify(humanName(name));
};

/**
 * Returns upper camel case
 * @param {String} name - name to convert
 * @return {String} - converted name
 */
var upperCamel = function upperCamel(name) {
  return _underscore2.default.classify(hyphenName(name));
};

module.exports = {
  humanName: humanName,
  hyphenName: hyphenName,
  upperCamel: upperCamel,

  /**
   * Returns controller name
   * @param {String} name - name to convert
   * @return {String} - converted name
   */
  ctrlName: function ctrlName(name) {
    return upperCamel(name) + 'Ctrl';
  },


  /**
   * Returns lower camel case
   * @param {String} name - name to convert
   * @return {String} - converted name
   */
  lowerCamel: function lowerCamel(name) {
    return _underscore2.default.camelize(hyphenName(name));
  }
};